// @(#)DrawView.h	2.21	10/29/91

#ifndef DrawView_First

#ifdef __GNUG__
#pragma once
#endif

#define DrawView_First

#include "OverView.h"
#include "ObjList.h"

class Shape;

class DrawView: public MapView {
    class TextShape	*activeText;
    class TextView	*activeTextView;
    Shape		*currShape;
    Shape		*gisShape;
    int			grid;
    Point		lastClick;   // position of last click
    bool		showGrid;
    ObjList		*shapes, *selection;
    bool		shapes_alloc;
    class OverView	*overView;
    class ArmySymbolDialog *armyDialog;

    void DrawOverlays(Rectangle);

    void DisplayCol(Rectangle r, class Collection *c);

public:
    MetaDef(DrawView);
    
    DrawView(class Document*, Point, ObjList*,
	float LongMin= 0, float LongMax= 0, float LatMin= 0, float LatMax= 0);
    ~DrawView();
    
    //---- initializing
    // void SetShapes(ObjList*);
    void ChangeShapes();
    ObjList *GetShapes()
	{ return shapes; }
    void SetOverView(OverView *ov)
	{ overView= ov; }
    OverView *GetOverView()
	{ return overView; }
    virtual ArmySymbolDialog *GetArmyDialog()
	{ return armyDialog; }
    void DoDynMenu(int p);
    
    //---- drawing
    virtual void TopDamaged();
    void Invalidate(ObjList *ol);
    void Draw(Rectangle);
    void NewContent();
    void EndGrab();
    void InitGrab();
    
    //---- shape list management
    void Insert(Shape *);
    void Remove(Shape *);
    void InsertShapes(ObjList*, bool tofront= TRUE);
    virtual Shape *FindShape(Point);
    virtual class QueryShape *FindQueryShape(Point, class TextItem **);
    Shape *FindHandle(Point p, int *nh);
    void SetGisShape(Shape *s) { gisShape= s; }

    // Geo ops
    virtual bool MakeNewPath(char *p);
    virtual bool EditPath(char *p);
    virtual bool MakeNewPoint(char *p);
         
    //---- misc
    void Point2Grid(Point *);
    void ConstrainScroll(Point *);
    Point LastClick()
	{ return lastClick; }
    void SetTool(Object*);
    void RequestTool(int tix);
    void ShowInfo(TrackPhase tp, char *fmt, ...);
    void SetMessage(char *m);

    //---- Selection
    int Selected()
	{ return selection->Size(); }
    Shape *OneSelected();
    Iterator *GetSelectionIter()
	{ return selection->MakeIterator(); }
    ObjList *GetCopyOfSelection();
    ObjList *GetDeepCopyOfSelection();
    ObjList *SetSelection(ObjList *newsel= 0);
    void SetDeleted(ObjList *l, bool b= TRUE);
    void RemoveDeleted();
    void ShowSelection();
    void SelectInRect(Rectangle r);
    Rectangle BoundingBox();
    bool HasSelection();
    
    //---- text
    void SetActiveText(class TextShape *tp);
    class TextShape *GetActiveText()
	{ return activeText; }
    
    //---- event handling
    Command *DoLeftButtonDownCommand(Point, Token, int);
    Command *DoMiddleButtonDownCommand(Point, Token, int);
    Command *DoKeyCommand(int, Point, Token);
    Command *DoCursorKeyCommand(EvtCursorDir, Point, Token);
    GrCursor GetCursor(Point lp);
    
    //---- menus
    void DoCreateMenu(class Menu*);
    void DoSetupMenu(class Menu*);
    Command *DoMenuCommand(int);
    
    //---- clipboard
#ifdef ET23
    bool CanPaste(Symbol type);
    void SelectionToClipboard(Symbol type, ostream &os);
    Command *PasteData(Symbol type, istream &is);
#else
    bool CanPaste(char * type);
    void SelectionToClipboard(char *type, ostream &os);
    Command *PasteData(char *type, istream &is);
#endif
};

#endif DrawView_First
