static char *sccs_id= (sccs_id, "%W% %G%");

/*
 * This file implements a GeoShape which draws maps which
 * are created with the rtree/makemaps script.
 */

#include "Class.h"
#include "Dictionary.h"
#include "ByteArray.h"
#include "ObjInt.h"
#include "OverView.h"
#include "IconShape.h"

#include <sys/file.h>
extern "C" {
#include <sys/mman.h>
}

// #define DEBUG

static char *GetMapData(ByteArray *mapname, long offset)
{
  static Dictionary	*mapfiles;
  static Set		*failed;
  ObjInt		*datap;
  char			*mp;

  if (mapfiles == 0) {
    mapfiles= new Dictionary;
    failed= new Set;
  }

  if (!(datap= (ObjInt *) mapfiles->AtKey(mapname))) {
	if (failed->Contains(mapname))
	  return 0;

	int	fdata;
	char	fn[1000];

	strcpy(fn, mapname->Str());
        if ((fdata = open(fn, O_RDONLY)) == -1) {
	  fprintf(stderr,
	      "Map data file: '%s' can't be opened!\n", fn);
	  failed->Add(mapname);
	  return 0;
	}

#if 1 // def DEBUG
fprintf(stderr, "Adding map %s\n", fn);
#endif
	long datasize=lseek(fdata,0L,L_XTND);
	if ((mp = mmap(0,datasize, PROT_READ, MAP_SHARED, fdata, 0))
           == (char *)-1) {
	    fprintf(stderr, "Map data file can't be mapped.\n");
	    perror(fn);
	    abort();
	}
	// mapfiles->AtKeyPut(mapname->DeepClone(), new ObjInt((int) mp));
	mapfiles->AtKeyPut(new ByteArray(mapname->Str()), new ObjInt((int) mp));
  } else {
	mp= (char*) int(*datap);
  }
  return mp+offset;
}



class MapShape: public PlineShape {
public:
  MapShape(MapView *, int n, char **str);
};


extern "C" MapShape *make_map_shape(MapView *dv, int n, char **str)
{
  MapShape *s= new MapShape(dv, n, str);
  return s;
}


MapShape::MapShape(MapView *d, int n, char **str): (d, 0)
{
    static ByteArray	*lastmap;
    static char		lastmapstr[200];
    const int		MaxPoints= 20000;

    if (lastmap == 0)
      lastmap= new ByteArray(str[2]);
    else if (strcmp(str[2], lastmapstr) != 0) {
      strcpy(lastmapstr, str[2]);
      delete lastmap;
      lastmap= new ByteArray(str[2]);
    }

    closed= str[0][0] == 't';

    short	*p= (short*) GetMapData(lastmap, atoi(str[1]));
    bool	fl= *p != -9999;

    p++;
    if (fl)
	p++;

    npoints= fl ? *(long*) p: *p;

#ifdef DEBUG
fprintf(stderr, "Npoints: %d\n", npoints);
#endif
    if (npoints >= MaxPoints) {
	fprintf(stderr, "MapShape: Too many points!\n");
	npoints= 0;
	return;
    }
    p++;
    if (fl)
	p++;

    Point	lastpoint, newpoint, tpts[MaxPoints];
    int		np= 0;
    for (int i= 0; i < npoints; i++) {
	float	Long, Lat;

	if (fl) {
	  Lat= (*(float*)p); p++; p++;
	  Long= (*(float*)p); p++; p++;
	} else {
	  Lat= (*p++)/float(100);
	  Long= (*p++)/float(100);
	}
	newpoint= dv->GeoToPoint(Long, Lat);
	if (newpoint == BADPOINT) {
	  np= 1;
	  break;
	}

	if (i == 0 || newpoint != lastpoint) {
	  lastpoint= tpts[np]= newpoint;
	  np++;
	}
    }

#ifdef DEBUG
fprintf(stderr, "Np: %d\n", np);
#endif
    Rectangle bbox;
    if (np == 1) {
      npoints= 0;
      pts= 0;
      bbox= Rectangle(tpts[0], gPoint0);
    } else {
      npoints= np;
      pts= new Point[npoints];
      bcopy(tpts, pts, npoints * sizeof(Point));
      bbox= NormRect(pts[0],pts[npoints-1]);
    }

    labelpos= bbox.Center();
    width= 1;
}
