//*****************************************************************************
// File: page.cc
// Implementation of the page manager.
//*****************************************************************************

#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include "page.h"

page_manager::page_manager(char *name, long low)
{
   PageName = new char[strlen(name)+1];
   sprintf(PageName, "%s", name);
   if ((free = new interval) == NULL) {
      fprintf(stderr, "class page_manager: Out of memory.\n");
      exit(1);
   }
   free->first = low;
   free->last = LONG_MAX;
   free->next = NULL;
}


page_manager::page_manager(char *name)
{
   int fd;
   interval *pres, *prev;
   long lower, upper, nbytes1, nbytes2;

   PageName = new char[strlen(name)+1];
   sprintf(PageName, "%s", name);
   if ((fd = open(PageName, O_RDONLY)) == -1) {
      fprintf(stderr, "class page_manager: Can't open %s.\n", PageName);
      exit(1);
   }
   pres = NULL;
   nbytes1 = read(fd, &lower, sizeof(long));
   if (nbytes1 == -1) {
      fprintf(stderr, "class page_manager: Read error.\n");
      exit(1);
   }
   nbytes2 = read(fd, &upper, sizeof(long));
   if (nbytes2 == -1) {
      fprintf(stderr, "class page_manager: Read error.\n");
      exit(1);
   }
   while (nbytes1 + nbytes2 == 2 * sizeof(long)) {
      prev = pres;
      if ((pres = new interval) == NULL) {
	 fprintf(stderr, "class page_manager: Out of memory.\n");
	 exit(1);
      }
      pres->first = lower;
      pres->last = upper;
      pres->next = NULL;
      if (prev == NULL)
	 free = pres;
      else
	 prev->next = pres;
      nbytes1 = read(fd, &lower, sizeof(long));
      if (nbytes1 == -1) {
	 fprintf(stderr, "class page_manager: Read error.\n");
	 exit(1);
      }
      nbytes2 = read(fd, &upper, sizeof(long));
      if (nbytes2 == -1) {
	 fprintf(stderr, "class page_manager: Read error.\n");
	 exit(1);
      }
   }
   if (nbytes1 + nbytes2 > 0) {
      fprintf(stderr, "class page_manager: Not enough data.\n");
      exit(1);
   }
   close(fd);
}


long page_manager::new_page()
{
   long avail_page;
   interval *temp;

   if (free != NULL) {
      avail_page = free->first++;
      if (free->first > free->last) {
	 temp = free;
	 free = free->next;
	 delete temp;
      }
   } else
      avail_page = NULL_PAGE;
   return avail_page;
}


void page_manager::dispose_page(long old_page)
{
   interval *pres, *prev, *temp;
   boolean found, ready;

   pres = free;
   prev = NULL;
   found = ready = false;
   while (!found && !ready) {
      if (pres == NULL)
	 ready = true;
      else if (old_page < pres->first - 1)
	 ready = true;
      else if ((old_page == pres->first - 1) ||
	       (old_page == pres->last + 1))
	 found = true;
      else {
	 prev = pres;
	 pres = pres->next;
      }
   }
   if (ready) {
      temp = new interval;
      temp->first = temp->last = old_page;
      temp->next = pres;
      if (prev != NULL)
	 prev->next = temp;
      else
	 free = temp;
   } else if (old_page == pres->first - 1) {
      pres->first--;
   } else {
      pres->last++;
      if ((temp = pres->next) != NULL)
	 if (pres->last + 1 == temp->first) {
	    pres->last = temp->last;
	    pres->next = temp->next;
	    delete temp;
	 }
   }
}


page_manager::~page_manager()
{
   interval *temp;
   int fd;

   if ((fd = creat(PageName, PERMS)) == -1) {
      fprintf(stderr, "class page_manager: Can't create %s.\n", PageName);
      exit(1);
   }
   while (free != NULL) {
      temp = free;
      free = free->next;
      write(fd, &(temp->first), sizeof(long));
      write(fd, &(temp->last), sizeof(long));
      delete temp;
   }
   close(fd);
   delete PageName;
}
